'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createFixturesFromDefinitions = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _faker = require('faker');

var _faker2 = _interopRequireDefault(_faker);

var _index = require('./index');

var _collection = require('./collection');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Create all fixtures from a given list of parsed swagger definitions
 * @param db A LokiJS DB instance
 * @param definitions A list of parsed swagger definitions
 */
var createFixturesFromDefinitions = exports.createFixturesFromDefinitions = function createFixturesFromDefinitions(db, definitions) {
    try {
        var createdModels = _lodash2.default.map(definitions, function (definition) {
            // store all models in datase
            return (0, _index.createModel)(db, definition);
        });

        // after all models are stored in database, we can start to generate fixtures for all
        // since we have all references stored

        return _lodash2.default.map(createdModels, function (model) {
            var fixtures = _lodash2.default.map(_lodash2.default.range(5), function (iter) {
                return generateOneFixture(db, model);
            });

            var modelName = model.modelName;

            var currentModelCollection = void 0;

            try {
                currentModelCollection = (0, _collection.findCollection)(db, modelName);
            } catch (e) {
                if (e.message === 'MODEL_DOES_NOT_EXIST') {
                    currentModelCollection = (0, _collection.createCollection)(db, modelName);
                }
            }

            return currentModelCollection.insert(fixtures);
        });
    } catch (e) {
        console.error(e);
    }
};

/**
 * Create one fixture from a given model
 * @param db Loki DB instance
 * @param model model content
 * @returns {{}}
 */
var generateOneFixture = function generateOneFixture(db, model) {
    var type = model.type,
        modelName = model.modelName;


    switch (type) {
        case 'object':
            var properties = model.properties;
            //handle dictionnaries (additonalProperties instead of properties)

            if (!properties) {
                var _fixture = {};

                //TODO handle maps or object with additionalProperties here

                return _fixture;
            } else {
                var _fixture2 = {};
                _fixture2[modelName] = _lodash2.default.reduce(properties, function (acc, prop, propName) {
                    return generate(db, acc, prop, propName, modelName);
                }, {});
                return _fixture2;
            }
            break;
        case 'array':
            var fixture = {};
            fixture[modelName] = _lodash2.default.map(_lodash2.default.range(5), function (elem) {
                return generate(db, {}, model.items, modelName)[modelName];
            });
            return fixture;
            break;
        default:
            throw new Error('Non handled generable type', { model: model, type: type });
            break;
    }
};

/**
 * Get reference model name from reference string
 * @param {string} item
 */
var cleanPropertyReference = function cleanPropertyReference(item) {
    return item.replace('#/definitions/', '');
};

/**
 * Generate one fixture
 * @param db
 * @param accumulator
 * @param property
 * @param propertyName
 * @param parentModelName
 * @returns {*}
 */
var generate = function generate(db, accumulator, property, propertyName, parentModelName) {
    accumulator[propertyName] = accumulator[propertyName] ? accumulator[propertyName] : {};
    var mapping = {
        'integer': _faker2.default.random.number,
        'number': _faker2.default.random.number,
        'string': _faker2.default.random.word,
        'boolean': _faker2.default.random.boolean

        //TODO make sure that we handle enum fields with non-random values

    };var customMapping = {
        'array': function array(property, propertyName) {
            var items = property.items;

            //handle case where array is an array of references to another object
            //we get the model from database and then generate from it;

            if (items['$ref']) {
                //TODO chain 5 times
                var modelName = cleanPropertyReference(items['$ref']);
                var model = (0, _index.getOneModel)(db, modelName);

                // if the current parsed model is the same than its container it is a circular reference
                if (modelName === parentModelName) {
                    return 'MORPHLING_CIRCULAR_REFERENCE';
                }

                return _lodash2.default.map(_lodash2.default.range(5), function (elem) {
                    return generateOneFixture(db, model);
                });
            }
            //else we just generate an array of fixtures
            return _lodash2.default.map(_lodash2.default.range(5), function (elem) {
                return generate(db, {}, items, propertyName)[propertyName];
            });
        },
        'lat': _faker2.default.address.latitude,
        'lng': _faker2.default.address.longitude,
        'zipcode': _faker2.default.address.zipCode,
        'address': _faker2.default.address.streetAddress
    };

    var fakerGenerators = (0, _extends3.default)({}, _faker2.default.random, _faker2.default.address, _faker2.default.company, _faker2.default.finance, _faker2.default.internet, _faker2.default.name, _faker2.default.random);

    //it is a reference to another object, keep reference for further generation down the line
    if (property['$ref']) {
        var modelName = cleanPropertyReference(property['$ref']);
        var model = (0, _index.getOneModel)(db, modelName);

        //console.log(generateOneFixture(db, model));
        accumulator[propertyName] = generateOneFixture(db, model);
    }
    // else a manual override exists for a given property
    else if (customMapping[propertyName]) {
            accumulator[propertyName] = customMapping[propertyName]();
        }
        //else a faker generator with the same name exists (using it will give more realistic data)
        else if (fakerGenerators[propertyName]) {
                accumulator[propertyName] = fakerGenerators[propertyName]();
            }
            //else use property type to generate the data
            else if (mapping[property.type]) {
                    accumulator[propertyName] = mapping[property.type]();
                }
                //else object is an array, we need to recure to generate
                else if (property.type === 'array') {
                        accumulator[propertyName] = customMapping['array'](property, propertyName);
                    }
                    //keep property as is if no field where generation could be done was found
                    else {
                            //TODO maybe example ?
                            accumulator[propertyName] = property;
                        }

    return accumulator;
};